@@ -12,7 +12,7 @@
 #' @importFrom utils combn
 #' @export
 CalculateTreeDistance <- function(Func, tree1, tree2 = NULL,
-                                   reportMatching = FALSE, ...) {
+                                  reportMatching = FALSE, ...) {
   supportedClasses <- c("phylo", "Splits")
   
   single1 <- inherits(tree1, supportedClasses)
@@ -20,8 +20,17 @@ CalculateTreeDistance <- function(Func, tree1, tree2 = NULL,
     stop("`tree1` must be a tree, or list of trees, with class ", 
          paste(supportedClasses, collapse = " / "))
   }
-  labels1 <- TipLabels(tree1, single = TRUE)
-  nTip <- length(labels1)
+  labels1 <- TipLabels(tree1)
+  if (is.list(labels1)) {
+    if (!all(vapply(labels1[-1], setequal, logical(1), labels1[[1]]))) {
+      nTip <- NA
+    } else {
+      labels1 <- labels1[[1]]
+      nTip <- length(labels1)
+    }
+  } else {
+    nTip <- length(labels1)
+  }
   
   null2 <- is.null(tree2)
   if (!null2) {
@@ -30,10 +39,17 @@ CalculateTreeDistance <- function(Func, tree1, tree2 = NULL,
       stop("`tree2` must be a tree, or list of trees, with class ", 
            paste(supportedClasses, collapse = " / "))
     }
-    labels2 <- TipLabels(tree2, single = TRUE)
+    labels2 <- TipLabels(tree2)
+    if (is.list(labels2)) {
+      if (!all(vapply(labels2[-1], setequal, logical(1), labels2[[1]]))) {
+        nTip <- NA
+      } else {
+        labels2 <- labels2[[1]]
+      }
+    }
     
-    if (length(setdiff(labels1, labels2)) > 0) {
-      stop("Leaves must bear identical labels.")
+    if (!setequal(labels1, labels2)) {
+      nTip <- NA
     }
   }
   
@@ -52,56 +68,103 @@ CalculateTreeDistance <- function(Func, tree1, tree2 = NULL,
       .SplitDistanceAllPairs(Func, tree1, tipLabels = labels1, nTip, ...)
     } else if (single2) {
       .SplitDistanceOneMany(Func, oneSplit = tree2, manySplits = tree1,
-                            tipLabels = labels2, ...)
+                            tipLabels = labels2, nTip = nTip, ...)
     } else {
-      .SplitDistanceManyMany(Func, tree1, tree2,
-                             tipLabels = labels1, ...)
+      .SplitDistanceManyMany(Func, tree1, tree2, tipLabels = labels1,
+                             nTip = nTip, ...)
     }
   }
 }
 
 .SplitDistanceOneOne <- function(Func, split1, split2, tipLabels, 
-                                 nTip = length(tipLabels), reportMatching, ...) {
-  Func(as.Splits(split1, asSplits = reportMatching),
-       as.Splits(split2, tipLabels = tipLabels, asSplits = reportMatching),
-       nTip = nTip, reportMatching = reportMatching, ...)
+                                 nTip = length(tipLabels), reportMatching,
+                                 ...) {
+  if (is.na(nTip)) {
+    common <- intersect(TipLabels(split1), TipLabels(split2))
+    split1 <- KeepTip(split1, common)
+    split2 <- KeepTip(split2, common)
+    nTip <- length(common)
+  }
+  if (nTip < 4) {
+    0
+  } else {
+    Func(as.Splits(split1, asSplits = reportMatching),
+         as.Splits(split2, tipLabels = tipLabels, asSplits = reportMatching),
+         nTip = nTip, reportMatching = reportMatching, ...)
+  }
 }
 
+#' @importFrom stats setNames
 .SplitDistanceOneMany <- function(Func, oneSplit, manySplits, 
                                   tipLabels, nTip = length(tipLabels), ...) {
-  s1 <- as.Splits(oneSplit, tipLabels = tipLabels, asSplits = FALSE)
-  vapply(manySplits,
-         function(s2) Func(s1, as.Splits(s2, tipLabels = tipLabels,
-                                          asSplits = FALSE),
-                            nTip = nTip, ...),
-         double(1))
+  if (is.na(nTip)) {
+    oneLabels <- TipLabels(oneSplit)
+    manyLabels <- TipLabels(manySplits)
+    common <- if (is.list(manyLabels)) {
+      lapply(manyLabels, intersect, oneLabels)
+    } else {
+      list(intersect(manyLabels, oneLabels))
+    }
+    setNames(unlist(.mapply(function(s2, keep) {
+      Func(
+        as.Splits(KeepTip(oneSplit, keep), tipLabels = keep, asSplits = FALSE),
+        as.Splits(KeepTip(s2, keep), tipLabels = keep, asSplits = FALSE),
+        nTip = length(keep),
+        ...
+      )
+    }, dots = list(manySplits, common), MoreArgs = NULL)), names(manySplits))
+  } else {
+    s1 <- as.Splits(oneSplit, tipLabels = tipLabels, asSplits = FALSE)
+    vapply(manySplits,
+           function(s2) Func(s1, as.Splits(s2, tipLabels = tipLabels,
+                                           asSplits = FALSE),
+                              nTip = nTip, ...),
+           double(1))
+  }
 }
 
 #' @importFrom parallel parCapply
 #' @importFrom cli cli_progress_bar cli_progress_update
 .SplitDistanceAllPairs <- function(Func, splits1, tipLabels,
-                                    nTip = length(tipLabels), ...) {
-  splits <- as.Splits(splits1, tipLabels = tipLabels, asSplits = FALSE)
+                                   nTip = length(tipLabels), ...) {
+  cluster <- getOption("TreeDist-cluster")
+
+  if (is.na(nTip)) {
+    splits <- lapply(splits1, as.Splits)
+    
+    .PairDist <- function(i) {
+      s1 <- splits[[i[1]]]
+      s2 <- splits[[i[2]]]
+      common <- intersect(TipLabels(s1), TipLabels(s2))
+      Func(KeepTip(s1, common), KeepTip(s2, common),
+           nTip = length(common), reportMatching = FALSE, ...)
+    }
+    
+  } else {
+    splits <- as.Splits(splits1, tipLabels = tipLabels, asSplits = FALSE)
+    
+    .PairDist <- function(i) {
+      Func(splits[[i[1]]], splits[[i[2]]],
+           nTip = nTip, reportMatching = FALSE, ...)
+    }
+    
+  }
+  
   nSplits <- length(splits)
   is <- combn(seq_len(nSplits), 2)
   
-  cluster <- getOption("TreeDist-cluster")
-  
   if (is.null(cluster)) {
     cli_progress_bar("Calculating distances", total = ncol(is))
   }
-  .PairDist <- function(i) {
-    Func(splits[[i[1]]], splits[[i[2]]],
-         nTip = nTip, reportMatching = FALSE, ...)
-  }
+  
   .CliPairDist <- function(i) { # TODO if cli possible from parallel, merge.
     cli_progress_update(1, .envir = parent.frame(2))
-    Func(splits[[i[1]]], splits[[i[2]]],
-         nTip = nTip, reportMatching = FALSE, ...)
+    .PairDist(i)
   }
   
   ret <- structure(class = "dist", Size = nSplits,
-                   diag = FALSE, upper = FALSE,
+                   Labels = names(splits1),
+                   Diag = FALSE, Upper = FALSE,
                    if (is.null(cluster)) {
                      apply(is, 2, .CliPairDist)
                    } else {
@@ -111,14 +174,35 @@ CalculateTreeDistance <- function(Func, tree1, tree2 = NULL,
   ret
 }
 
+#' @importFrom stats setNames
 .SplitDistanceManyMany <- function(Func, splits1, splits2, 
-                                    tipLabels, nTip = length(tipLabels), ...) {
-  splits1 <- as.Splits(splits1, tipLabels = tipLabels, asSplits = FALSE)
-  splits2 <- as.Splits(splits2, tipLabels = tipLabels, asSplits = FALSE)
-  matrix(mapply(Func, rep(splits2, each = length(splits1)), splits1,
-                nTip = nTip, ...),
-         length(splits1), length(splits2),
-         dimnames = list(names(splits1), names(splits2)))
+                                   tipLabels, nTip = length(tipLabels), ...) {
+  
+  if (is.na(nTip)) {
+    tipLabels <- union(unlist(tipLabels), unlist(TipLabels(splits2)))
+    splits1 <- as.Splits(splits1, tipLabels = tipLabels, asSplits = TRUE)
+    splits2 <- as.Splits(splits2, tipLabels = tipLabels, asSplits = TRUE)
+    vapply(splits1, function(s1) {
+        l1 <- TipLabels(s1)
+        vapply(splits2, function(s2) {
+          common <- intersect(l1, TipLabels(s2))
+          Func(KeepTip(s1, common), KeepTip(s2, common),
+               nTip = length(common))#, ...)
+        }, double(1))
+      },
+      setNames(double(length(splits2)), names(splits2))
+    )
+  } else {
+    splits1 <- as.Splits(splits1, tipLabels = tipLabels, asSplits = FALSE)
+    splits2 <- as.Splits(splits2, tipLabels = tipLabels, asSplits = FALSE)
+    matrix(
+      unlist(.mapply(Func, list(rep(splits2, each = length(splits1)),
+                         splits1, nTip = nTip, ...), NULL)),
+      length(splits1),
+      length(splits2),
+      dimnames = list(names(splits1), names(splits2))
+    )
+  }
 }
 
 #' Calculate distance between trees, or lists of trees
@@ -329,44 +413,137 @@ CompareAll <- function(x, Func, FUN.VALUE = Func(x[[1]], x[[1]], ...),
 #' @param \dots Additional parameters to `InfoInTree()` or `how`.
 #' @keywords internal
 #' @template MRS
+#' @importFrom TreeTools KeepTip TipLabels
 #' @export
-NormalizeInfo <- function(unnormalized, tree1, tree2, InfoInTree, 
-                           infoInBoth = NULL,
-                           how = TRUE, Combine = "+", ...) {
+NormalizeInfo <- function(unnormalized, tree1, tree2, InfoInTree,
+                          infoInBoth = NULL, how = TRUE, Combine = "+", ...) {
   
-  CombineInfo <- function(tree1Info, tree2Info, Combiner = Combine) {
-    if (length(tree1Info) == 1 || length(tree2Info) == 1) {
-      mapply(Combiner, tree1Info, tree2Info)
+  CombineInfo <- function(tree1Info, tree2Info, Combiner = Combine,
+                          pairwise = FALSE) {
+    if (length(tree1Info) == 1 || length(tree2Info) == 1 || pairwise) {
+      unlist(.mapply(Combiner, dots = list(tree1Info, tree2Info), NULL))
     } else {
       ret <- outer(tree1Info, tree2Info, Combiner)
       if (inherits(unnormalized, "dist")) ret[lower.tri(ret)] else ret
     }
   }
   
+  lab1 <- TipLabels(tree1)
+  lab2 <- TipLabels(tree2)
+  sameLabels <- .AllTipsSame(lab1, lab2)
+  
+  if (!sameLabels) {
+    trees <- .SharedOnly(tree1, tree2, lab1, lab2)
+    tree1 <- trees[[1]]
+    tree2 <- trees[[2]]
+  }
+  
   if (is.logical(how)) {
     if (how == FALSE) {
-      return (unnormalized)
+      return(unnormalized)
     } else {
       if (is.null(infoInBoth)) {
         info1 <- InfoInTree(tree1, ...)
-        info2 <- if (is.null(tree2)) info1 else InfoInTree(tree2, ...)
-        infoInBoth <- CombineInfo(info1, info2)
+        info2 <- if (is.null(tree2)) {
+          info1
+        } else {
+          InfoInTree(tree2, ...)
+        }
+        infoInBoth <- CombineInfo(info1, info2, pairwise = !sameLabels)
       }
     }
   } else if (is.function(how)) {
     if (is.null(infoInBoth)) {
       info1 <- InfoInTree(tree1, ...)
       info2 <- if (is.null(tree2)) info1 else InfoInTree(tree2, ...)
-      infoInBoth <- CombineInfo(info1, info2, Combiner = how)
+      infoInBoth <- CombineInfo(info1, info2, Combiner = how,
+                                pairwise = !sameLabels)
     }
   } else {
     infoInBoth <- how
   }
-  
   # Return:
   unnormalized / infoInBoth
 }
 
+# We only call this function when not all trees contain identical leaf sets
+#' @importFrom TreeTools KeepTip TipLabels
+.SharedOnly <- function(tree1, tree2,
+                        lab1 = TipLabels(tree1),
+                        lab2 = TipLabels(tree2)) {
+  if (is.null(tree2)) {
+    # Case: N trees vs themselves
+    # We require a triangular matrix suitable for a dist object
+    if (.MultipleTrees(tree1)) {
+      pairs <- combn(seq_along(tree1), 2)
+      nPairs <- dim(pairs)[2]
+      ret <- list(vector("list", nPairs), vector("list", nPairs))
+      
+      for (n in seq_len(nPairs)) {
+        i <- pairs[1, n]
+        j <- pairs[2, n]
+        common <- intersect(lab1[[i]], lab1[[j]])
+        ret[[1]][[n]] <- KeepTip(tree1[[i]], common)
+        ret[[2]][[n]] <- KeepTip(tree1[[j]], common)
+      }
+      
+      # Return:
+      ret
+    } else {
+      return(list(NULL, NULL))
+    }
+  } else if (.MultipleTrees(tree1)) {
+    if (.MultipleTrees(tree2)) {
+      # Case: N vs M
+      n1 <- length(tree1)
+      n2 <- length(tree2)
+      nPairs <- n1 * n2
+      ret <- list(vector("list", nPairs), vector("list", nPairs))
+      pair1 <- rep(tree1, each = n2)
+      pair2 <- rep.int(tree2, times = n1)
+      lab1 <- if (is.list(lab1)) {
+        rep(lab1, each = n2)
+      } else {
+        rep.int(list(lab1), times = n1 * n2)
+      }
+      lab2 <- if (is.list(lab2)) {
+        rep.int(lab2, times = n1)
+      } else {
+        rep.int(list(lab2), times = n1 * n2)
+      }
+      for (i in seq_len(nPairs)) {
+        common <- intersect(lab1[[i]], lab2[[i]])
+        ret[[1]][[i]] <- KeepTip(pair1[[i]], common)
+        ret[[2]][[i]] <- KeepTip(pair2[[i]], common)
+      }
+      # Return:
+      ret
+    } else {
+      # Case: N vs 1
+      commonLeaves <- lapply(if (is.list(lab1)) lab1 else list(lab1),
+                             intersect, lab2)
+      list(.mapply(KeepTip, list(tree1, commonLeaves), NULL),
+           lapply(commonLeaves, function (common) KeepTip(tree2, common)))
+    }
+  } else {
+    if (.MultipleTrees(tree2)) {
+      # Case: 1 vs N
+      commonLeaves <- lapply(if (is.list(lab2)) lab2 else list(lab2),
+                             intersect, lab1)
+      list(lapply(commonLeaves, function (common) KeepTip(tree1, common)),
+           .mapply(KeepTip, list(tree2, commonLeaves), NULL))
+    } else {
+      # Case: 1 vs 1
+      commonLeaves <- intersect(lab1, lab2)
+      list(KeepTip(tree1, commonLeaves), KeepTip(tree2, commonLeaves))
+    }
+  }
+}
+
+.MultipleTrees <- function(tree) {
+  is.list(tree) && inherits(tree[[1]], "phylo") && length(tree) > 1
+}
+
 #' List clades as text
 #' @param splits,splits1,splits2 Logical matrices with columns specifying membership
 #' of each corresponding matched clade.
