@@ -11,7 +11,8 @@ test_that("Split info calculated", {
   
   trees[] <- lapply(trees, RenumberTips, c(letters[1:7], "x", "z"))
   expect_equal(consensus_info(trees, TRUE, 1), 0)
-  expect_equal(SplitwiseInfo(consensus(trees, p = 0.5), p = split_p),
+  expect_equal(SplitwiseInfo(consensus(trees, p = 0.5, rooted = TRUE),
+                             p = split_p),
                ConsensusInfo(trees, p = 0.5))
   expect_warning(expect_equal(ConsensusInfo(trees, p = 0.4),
                               ConsensusInfo(trees, p = 0.5)))
